# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from urllib.parse import urlencode, urlparse, urlunparse

from aiohttp.client import _WSRequestContextManager
from multidict import CIMultiDict, CIMultiDictProxy

from kubernetes_asyncio.client import ApiClient
from kubernetes_asyncio.client.configuration import Configuration
from kubernetes_asyncio.client.rest import RESTResponse

STDIN_CHANNEL = 0
STDOUT_CHANNEL = 1
STDERR_CHANNEL = 2
ERROR_CHANNEL = 3
RESIZE_CHANNEL = 4


def get_websocket_url(url: str) -> str:
    parsed_url = urlparse(url)
    parts = list(parsed_url)
    if parsed_url.scheme == "http":
        parts[0] = "ws"
    elif parsed_url.scheme == "https":
        parts[0] = "wss"
    return urlunparse(parts)


class WsResponse(RESTResponse):
    def __init__(self, status, data) -> None:
        self.status = status
        self.data = data
        self.reason = ""

    def getheaders(self) -> CIMultiDictProxy:
        return CIMultiDictProxy(CIMultiDict())

    def getheader(self, name: str, default: str | None = None) -> str | None:
        return None


class WsApiClient(ApiClient):
    def __init__(
        self,
        configuration: Configuration | None = None,
        header_name: str | None = None,
        header_value: str | None = None,
        cookie: str | None = None,
        pool_threads: int = 1,
        heartbeat: float | None = None,
    ) -> None:
        super().__init__(configuration, header_name, header_value, cookie, pool_threads)
        self.heartbeat = heartbeat

    @classmethod
    def parse_error_data(cls, error_data: str | bytes) -> int:
        """
        Parse data received on ERROR_CHANNEL and return the command exit code.
        """
        error_data_json = json.loads(error_data)
        if error_data_json.get("status") == "Success":
            return 0
        return int(error_data_json["details"]["causes"][0]["message"])

    async def request(
        self,
        method,
        url,
        query_params=None,
        headers=None,
        post_params=None,
        body=None,
        _preload_content=True,
        _request_timeout=None,
    ) -> WsResponse | _WSRequestContextManager:
        # Expand command parameter list to indivitual command params
        if query_params:
            new_query_params = []
            for key, value in query_params:
                if key == "command" and isinstance(value, list):
                    for command in value:
                        new_query_params.append((key, command))
                else:
                    new_query_params.append((key, value))
            query_params = new_query_params

        if headers is None:
            headers = {}
        if "sec-websocket-protocol" not in headers:
            headers["sec-websocket-protocol"] = "v4.channel.k8s.io"

        if query_params:
            url += "?" + urlencode(query_params)

        url = get_websocket_url(url)

        if _preload_content:
            resp_all = ""
            async with self.rest_client.pool_manager.ws_connect(
                url, headers=headers, heartbeat=self.heartbeat
            ) as ws:
                async for wsmsg in ws:
                    msg = wsmsg.data.decode("utf-8")
                    if len(msg) > 1:
                        channel = ord(msg[0])
                        data = msg[1:]
                        if data:
                            if channel in [STDOUT_CHANNEL, STDERR_CHANNEL]:
                                resp_all += data

            return WsResponse(200, resp_all.encode("utf-8"))

        else:
            return self.rest_client.pool_manager.ws_connect(
                url, headers=headers, heartbeat=self.heartbeat
            )



from kubernetes_asyncio.client.models import V1NodeSwapStatus
from typing import Any

class V1NodeSystemInfo:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, architecture: str | None = None, boot_id: str | None = None, container_runtime_version: str | None = None, kernel_version: str | None = None, kube_proxy_version: str | None = None, kubelet_version: str | None = None, machine_id: str | None = None, operating_system: str | None = None, os_image: str | None = None, swap: V1NodeSwapStatus | None = None, system_uuid: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def architecture(self) -> str: ...
    @architecture.setter
    def architecture(self, architecture: str) -> None: ...
    @property
    def boot_id(self) -> str: ...
    @boot_id.setter
    def boot_id(self, boot_id: str) -> None: ...
    @property
    def container_runtime_version(self) -> str: ...
    @container_runtime_version.setter
    def container_runtime_version(self, container_runtime_version: str) -> None: ...
    @property
    def kernel_version(self) -> str: ...
    @kernel_version.setter
    def kernel_version(self, kernel_version: str) -> None: ...
    @property
    def kube_proxy_version(self) -> str: ...
    @kube_proxy_version.setter
    def kube_proxy_version(self, kube_proxy_version: str) -> None: ...
    @property
    def kubelet_version(self) -> str: ...
    @kubelet_version.setter
    def kubelet_version(self, kubelet_version: str) -> None: ...
    @property
    def machine_id(self) -> str: ...
    @machine_id.setter
    def machine_id(self, machine_id: str) -> None: ...
    @property
    def operating_system(self) -> str: ...
    @operating_system.setter
    def operating_system(self, operating_system: str) -> None: ...
    @property
    def os_image(self) -> str: ...
    @os_image.setter
    def os_image(self, os_image: str) -> None: ...
    @property
    def swap(self) -> V1NodeSwapStatus: ...
    @swap.setter
    def swap(self, swap: V1NodeSwapStatus) -> None: ...
    @property
    def system_uuid(self) -> str: ...
    @system_uuid.setter
    def system_uuid(self, system_uuid: str) -> None: ...



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1.34.3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from kubernetes_asyncio.client.configuration import Configuration


class V1PodCondition(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'last_probe_time': 'datetime',
        'last_transition_time': 'datetime',
        'message': 'str',
        'observed_generation': 'int',
        'reason': 'str',
        'status': 'str',
        'type': 'str'
    }

    attribute_map = {
        'last_probe_time': 'lastProbeTime',
        'last_transition_time': 'lastTransitionTime',
        'message': 'message',
        'observed_generation': 'observedGeneration',
        'reason': 'reason',
        'status': 'status',
        'type': 'type'
    }

    def __init__(self, last_probe_time=None, last_transition_time=None, message=None, observed_generation=None, reason=None, status=None, type=None, local_vars_configuration=None):  # noqa: E501
        """V1PodCondition - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default()
        self.local_vars_configuration = local_vars_configuration

        self._last_probe_time = None
        self._last_transition_time = None
        self._message = None
        self._observed_generation = None
        self._reason = None
        self._status = None
        self._type = None
        self.discriminator = None

        if last_probe_time is not None:
            self.last_probe_time = last_probe_time
        if last_transition_time is not None:
            self.last_transition_time = last_transition_time
        if message is not None:
            self.message = message
        if observed_generation is not None:
            self.observed_generation = observed_generation
        if reason is not None:
            self.reason = reason
        self.status = status
        self.type = type

    @property
    def last_probe_time(self):
        """Gets the last_probe_time of this V1PodCondition.  # noqa: E501

        Last time we probed the condition.  # noqa: E501

        :return: The last_probe_time of this V1PodCondition.  # noqa: E501
        :rtype: datetime
        """
        return self._last_probe_time

    @last_probe_time.setter
    def last_probe_time(self, last_probe_time):
        """Sets the last_probe_time of this V1PodCondition.

        Last time we probed the condition.  # noqa: E501

        :param last_probe_time: The last_probe_time of this V1PodCondition.  # noqa: E501
        :type last_probe_time: datetime
        """

        self._last_probe_time = last_probe_time

    @property
    def last_transition_time(self):
        """Gets the last_transition_time of this V1PodCondition.  # noqa: E501

        Last time the condition transitioned from one status to another.  # noqa: E501

        :return: The last_transition_time of this V1PodCondition.  # noqa: E501
        :rtype: datetime
        """
        return self._last_transition_time

    @last_transition_time.setter
    def last_transition_time(self, last_transition_time):
        """Sets the last_transition_time of this V1PodCondition.

        Last time the condition transitioned from one status to another.  # noqa: E501

        :param last_transition_time: The last_transition_time of this V1PodCondition.  # noqa: E501
        :type last_transition_time: datetime
        """

        self._last_transition_time = last_transition_time

    @property
    def message(self):
        """Gets the message of this V1PodCondition.  # noqa: E501

        Human-readable message indicating details about last transition.  # noqa: E501

        :return: The message of this V1PodCondition.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this V1PodCondition.

        Human-readable message indicating details about last transition.  # noqa: E501

        :param message: The message of this V1PodCondition.  # noqa: E501
        :type message: str
        """

        self._message = message

    @property
    def observed_generation(self):
        """Gets the observed_generation of this V1PodCondition.  # noqa: E501

        If set, this represents the .metadata.generation that the pod condition was set based upon. This is an alpha field. Enable PodObservedGenerationTracking to be able to use this field.  # noqa: E501

        :return: The observed_generation of this V1PodCondition.  # noqa: E501
        :rtype: int
        """
        return self._observed_generation

    @observed_generation.setter
    def observed_generation(self, observed_generation):
        """Sets the observed_generation of this V1PodCondition.

        If set, this represents the .metadata.generation that the pod condition was set based upon. This is an alpha field. Enable PodObservedGenerationTracking to be able to use this field.  # noqa: E501

        :param observed_generation: The observed_generation of this V1PodCondition.  # noqa: E501
        :type observed_generation: int
        """

        self._observed_generation = observed_generation

    @property
    def reason(self):
        """Gets the reason of this V1PodCondition.  # noqa: E501

        Unique, one-word, CamelCase reason for the condition's last transition.  # noqa: E501

        :return: The reason of this V1PodCondition.  # noqa: E501
        :rtype: str
        """
        return self._reason

    @reason.setter
    def reason(self, reason):
        """Sets the reason of this V1PodCondition.

        Unique, one-word, CamelCase reason for the condition's last transition.  # noqa: E501

        :param reason: The reason of this V1PodCondition.  # noqa: E501
        :type reason: str
        """

        self._reason = reason

    @property
    def status(self):
        """Gets the status of this V1PodCondition.  # noqa: E501

        Status is the status of the condition. Can be True, False, Unknown. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle#pod-conditions  # noqa: E501

        :return: The status of this V1PodCondition.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this V1PodCondition.

        Status is the status of the condition. Can be True, False, Unknown. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle#pod-conditions  # noqa: E501

        :param status: The status of this V1PodCondition.  # noqa: E501
        :type status: str
        """
        if self.local_vars_configuration.client_side_validation and status is None:  # noqa: E501
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501

        self._status = status

    @property
    def type(self):
        """Gets the type of this V1PodCondition.  # noqa: E501

        Type is the type of the condition. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle#pod-conditions  # noqa: E501

        :return: The type of this V1PodCondition.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this V1PodCondition.

        Type is the type of the condition. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle#pod-conditions  # noqa: E501

        :param type: The type of this V1PodCondition.  # noqa: E501
        :type type: str
        """
        if self.local_vars_configuration.client_side_validation and type is None:  # noqa: E501
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501

        self._type = type

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1PodCondition):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1PodCondition):
            return True

        return self.to_dict() != other.to_dict()



from kubernetes_asyncio.client.models import V1alpha1MatchResources
from kubernetes_asyncio.client.models import V1alpha1ParamRef
from typing import Any

class V1alpha1MutatingAdmissionPolicyBindingSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, match_resources: V1alpha1MatchResources | None = None, param_ref: V1alpha1ParamRef | None = None, policy_name: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def match_resources(self) -> V1alpha1MatchResources: ...
    @match_resources.setter
    def match_resources(self, match_resources: V1alpha1MatchResources) -> None: ...
    @property
    def param_ref(self) -> V1alpha1ParamRef: ...
    @param_ref.setter
    def param_ref(self, param_ref: V1alpha1ParamRef) -> None: ...
    @property
    def policy_name(self) -> str: ...
    @policy_name.setter
    def policy_name(self, policy_name: str) -> None: ...



# Copyright 2019 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import IsolatedAsyncioTestCase
from unittest.mock import AsyncMock

from kubernetes_asyncio.utils import create_from_dict, create_from_yaml


class CreateFromYamlTest(IsolatedAsyncioTestCase):
    async def test_create_from_yaml(self) -> None:
        api_client = AsyncMock()
        api_client.call_api = AsyncMock()
        api_client.call_api.return_value = "mock-value"

        created = await create_from_yaml(api_client, "examples/nginx-deployment.yaml")

        # simple check for api call
        self.assertEqual(
            api_client.call_api.call_args[0][0],
            "/apis/apps/v1/namespaces/{namespace}/deployments",
        )

        # returned values
        self.assertEqual(created, [["mock-value"]])

    async def test_create_from_dict(self) -> None:
        api_client = AsyncMock()
        api_client.call_api = AsyncMock()
        api_client.call_api.return_value = "mock-value"

        created = await create_from_dict(
            api_client,
            {
                "apiVersion": "apps/v1",
                "kind": "Deployment",
                "metadata": {"name": "nginx-deployment"},
                "spec": {
                    "replicas": 3,
                },
            },
        )

        # simple check for api call
        self.assertEqual(
            api_client.call_api.call_args[0][0],
            "/apis/apps/v1/namespaces/{namespace}/deployments",
        )

        # returned values
        self.assertEqual(created, ["mock-value"])



from kubernetes_asyncio.client.models import CoreV1EndpointPort
from kubernetes_asyncio.client.models import V1EndpointAddress
from typing import Any

class V1EndpointSubset:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, addresses: list[V1EndpointAddress] | None = None, not_ready_addresses: list[V1EndpointAddress] | None = None, ports: list[CoreV1EndpointPort] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def addresses(self) -> list[V1EndpointAddress]: ...
    @addresses.setter
    def addresses(self, addresses: list[V1EndpointAddress]) -> None: ...
    @property
    def not_ready_addresses(self) -> list[V1EndpointAddress]: ...
    @not_ready_addresses.setter
    def not_ready_addresses(self, not_ready_addresses: list[V1EndpointAddress]) -> None: ...
    @property
    def ports(self) -> list[CoreV1EndpointPort]: ...
    @ports.setter
    def ports(self, ports: list[CoreV1EndpointPort]) -> None: ...



from kubernetes_asyncio.client.models import V1ConfigMapNodeConfigSource
from typing import Any

class V1NodeConfigSource:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, config_map: V1ConfigMapNodeConfigSource | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def config_map(self) -> V1ConfigMapNodeConfigSource: ...
    @config_map.setter
    def config_map(self, config_map: V1ConfigMapNodeConfigSource) -> None: ...



from kubernetes_asyncio.client.models import V1VolumeError
from typing import Any

class V1VolumeAttachmentStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, attach_error: V1VolumeError | None = None, attached: bool | None = None, attachment_metadata: dict[str, str] | None = None, detach_error: V1VolumeError | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def attach_error(self) -> V1VolumeError: ...
    @attach_error.setter
    def attach_error(self, attach_error: V1VolumeError) -> None: ...
    @property
    def attached(self) -> bool: ...
    @attached.setter
    def attached(self, attached: bool) -> None: ...
    @property
    def attachment_metadata(self) -> dict[str, str]: ...
    @attachment_metadata.setter
    def attachment_metadata(self, attachment_metadata: dict[str, str]) -> None: ...
    @property
    def detach_error(self) -> V1VolumeError: ...
    @detach_error.setter
    def detach_error(self, detach_error: V1VolumeError) -> None: ...



from typing import Any

class V1LinuxContainerUser:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, gid: int | None = None, supplemental_groups: list[int] | None = None, uid: int | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def gid(self) -> int: ...
    @gid.setter
    def gid(self, gid: int) -> None: ...
    @property
    def supplemental_groups(self) -> list[int]: ...
    @supplemental_groups.setter
    def supplemental_groups(self, supplemental_groups: list[int]) -> None: ...
    @property
    def uid(self) -> int: ...
    @uid.setter
    def uid(self, uid: int) -> None: ...



from datetime import datetime
from typing import Any

class V1HorizontalPodAutoscalerStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, current_cpu_utilization_percentage: int | None = None, current_replicas: int | None = None, desired_replicas: int | None = None, last_scale_time: datetime | None = None, observed_generation: int | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def current_cpu_utilization_percentage(self) -> int: ...
    @current_cpu_utilization_percentage.setter
    def current_cpu_utilization_percentage(self, current_cpu_utilization_percentage: int) -> None: ...
    @property
    def current_replicas(self) -> int: ...
    @current_replicas.setter
    def current_replicas(self, current_replicas: int) -> None: ...
    @property
    def desired_replicas(self) -> int: ...
    @desired_replicas.setter
    def desired_replicas(self, desired_replicas: int) -> None: ...
    @property
    def last_scale_time(self) -> datetime: ...
    @last_scale_time.setter
    def last_scale_time(self, last_scale_time: datetime) -> None: ...
    @property
    def observed_generation(self) -> int: ...
    @observed_generation.setter
    def observed_generation(self, observed_generation: int) -> None: ...



from kubernetes_asyncio.client.models import V1CSINodeDriver
from typing import Any

class V1CSINodeSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, drivers: list[V1CSINodeDriver] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def drivers(self) -> list[V1CSINodeDriver]: ...
    @drivers.setter
    def drivers(self, drivers: list[V1CSINodeDriver]) -> None: ...



from kubernetes_asyncio.client.models import V1IngressBackend
from typing import Any

class V1HTTPIngressPath:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, backend: V1IngressBackend | None = None, path: str | None = None, path_type: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def backend(self) -> V1IngressBackend: ...
    @backend.setter
    def backend(self, backend: V1IngressBackend) -> None: ...
    @property
    def path(self) -> str: ...
    @path.setter
    def path(self, path: str) -> None: ...
    @property
    def path_type(self) -> str: ...
    @path_type.setter
    def path_type(self, path_type: str) -> None: ...



from datetime import datetime
from typing import Any

class V1TokenRequestStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, expiration_timestamp: datetime | None = None, token: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def expiration_timestamp(self) -> datetime: ...
    @expiration_timestamp.setter
    def expiration_timestamp(self, expiration_timestamp: datetime) -> None: ...
    @property
    def token(self) -> str: ...
    @token.setter
    def token(self, token: str) -> None: ...



from kubernetes_asyncio.client.api_client import ApiClient
from kubernetes_asyncio.client.models import V1APIResourceList
from kubernetes_asyncio.client.models import V1SelfSubjectReview
from kubernetes_asyncio.client.models import V1TokenReview
from multidict import CIMultiDictProxy
from typing import Any
from typing import Awaitable

class AuthenticationV1Api:
    def __init__(self, api_client: ApiClient | None = None) -> None: ...
    def create_self_subject_review(self, body: V1SelfSubjectReview, **kwargs: Any) -> Awaitable[V1SelfSubjectReview]: ...
    def create_self_subject_review_with_http_info(self, body: V1SelfSubjectReview, **kwargs: Any) -> Awaitable[tuple[V1SelfSubjectReview, int, CIMultiDictProxy]]: ...
    def create_token_review(self, body: V1TokenReview, **kwargs: Any) -> Awaitable[V1TokenReview]: ...
    def create_token_review_with_http_info(self, body: V1TokenReview, **kwargs: Any) -> Awaitable[tuple[V1TokenReview, int, CIMultiDictProxy]]: ...
    def get_api_resources(self, **kwargs: Any) -> Awaitable[V1APIResourceList]: ...
    def get_api_resources_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1APIResourceList, int, CIMultiDictProxy]]: ...



from kubernetes_asyncio.client.models import V1ExecAction
from kubernetes_asyncio.client.models import V1GRPCAction
from kubernetes_asyncio.client.models import V1HTTPGetAction
from kubernetes_asyncio.client.models import V1TCPSocketAction
from typing import Any

class V1Probe:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, _exec: V1ExecAction | None = None, failure_threshold: int | None = None, grpc: V1GRPCAction | None = None, http_get: V1HTTPGetAction | None = None, initial_delay_seconds: int | None = None, period_seconds: int | None = None, success_threshold: int | None = None, tcp_socket: V1TCPSocketAction | None = None, termination_grace_period_seconds: int | None = None, timeout_seconds: int | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def _exec(self) -> V1ExecAction: ...
    @_exec.setter
    def _exec(self, _exec: V1ExecAction) -> None: ...
    @property
    def failure_threshold(self) -> int: ...
    @failure_threshold.setter
    def failure_threshold(self, failure_threshold: int) -> None: ...
    @property
    def grpc(self) -> V1GRPCAction: ...
    @grpc.setter
    def grpc(self, grpc: V1GRPCAction) -> None: ...
    @property
    def http_get(self) -> V1HTTPGetAction: ...
    @http_get.setter
    def http_get(self, http_get: V1HTTPGetAction) -> None: ...
    @property
    def initial_delay_seconds(self) -> int: ...
    @initial_delay_seconds.setter
    def initial_delay_seconds(self, initial_delay_seconds: int) -> None: ...
    @property
    def period_seconds(self) -> int: ...
    @period_seconds.setter
    def period_seconds(self, period_seconds: int) -> None: ...
    @property
    def success_threshold(self) -> int: ...
    @success_threshold.setter
    def success_threshold(self, success_threshold: int) -> None: ...
    @property
    def tcp_socket(self) -> V1TCPSocketAction: ...
    @tcp_socket.setter
    def tcp_socket(self, tcp_socket: V1TCPSocketAction) -> None: ...
    @property
    def termination_grace_period_seconds(self) -> int: ...
    @termination_grace_period_seconds.setter
    def termination_grace_period_seconds(self, termination_grace_period_seconds: int) -> None: ...
    @property
    def timeout_seconds(self) -> int: ...
    @timeout_seconds.setter
    def timeout_seconds(self, timeout_seconds: int) -> None: ...



from kubernetes_asyncio.client.api_client import ApiClient
from kubernetes_asyncio.client.models import EventsV1Event
from kubernetes_asyncio.client.models import EventsV1EventList
from kubernetes_asyncio.client.models import V1APIResourceList
from kubernetes_asyncio.client.models import V1Status
from multidict import CIMultiDictProxy
from typing import Any
from typing import Awaitable

class EventsV1Api:
    def __init__(self, api_client: ApiClient | None = None) -> None: ...
    def create_namespaced_event(self, namespace: str, body: EventsV1Event, **kwargs: Any) -> Awaitable[EventsV1Event]: ...
    def create_namespaced_event_with_http_info(self, namespace: str, body: EventsV1Event, **kwargs: Any) -> Awaitable[tuple[EventsV1Event, int, CIMultiDictProxy]]: ...
    def delete_collection_namespaced_event(self, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_namespaced_event_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_namespaced_event(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_namespaced_event_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def get_api_resources(self, **kwargs: Any) -> Awaitable[V1APIResourceList]: ...
    def get_api_resources_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1APIResourceList, int, CIMultiDictProxy]]: ...
    def list_event_for_all_namespaces(self, **kwargs: Any) -> Awaitable[EventsV1EventList]: ...
    def list_event_for_all_namespaces_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[EventsV1EventList, int, CIMultiDictProxy]]: ...
    def list_namespaced_event(self, namespace: str, **kwargs: Any) -> Awaitable[EventsV1EventList]: ...
    def list_namespaced_event_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[EventsV1EventList, int, CIMultiDictProxy]]: ...
    def patch_namespaced_event(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[EventsV1Event]: ...
    def patch_namespaced_event_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[EventsV1Event, int, CIMultiDictProxy]]: ...
    def read_namespaced_event(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[EventsV1Event]: ...
    def read_namespaced_event_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[EventsV1Event, int, CIMultiDictProxy]]: ...
    def replace_namespaced_event(self, name: str, namespace: str, body: EventsV1Event, **kwargs: Any) -> Awaitable[EventsV1Event]: ...
    def replace_namespaced_event_with_http_info(self, name: str, namespace: str, body: EventsV1Event, **kwargs: Any) -> Awaitable[tuple[EventsV1Event, int, CIMultiDictProxy]]: ...



from kubernetes_asyncio.client.models import V1CustomResourceConversion
from kubernetes_asyncio.client.models import V1CustomResourceDefinitionNames
from kubernetes_asyncio.client.models import V1CustomResourceDefinitionVersion
from typing import Any

class V1CustomResourceDefinitionSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, conversion: V1CustomResourceConversion | None = None, group: str | None = None, names: V1CustomResourceDefinitionNames | None = None, preserve_unknown_fields: bool | None = None, scope: str | None = None, versions: list[V1CustomResourceDefinitionVersion] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def conversion(self) -> V1CustomResourceConversion: ...
    @conversion.setter
    def conversion(self, conversion: V1CustomResourceConversion) -> None: ...
    @property
    def group(self) -> str: ...
    @group.setter
    def group(self, group: str) -> None: ...
    @property
    def names(self) -> V1CustomResourceDefinitionNames: ...
    @names.setter
    def names(self, names: V1CustomResourceDefinitionNames) -> None: ...
    @property
    def preserve_unknown_fields(self) -> bool: ...
    @preserve_unknown_fields.setter
    def preserve_unknown_fields(self, preserve_unknown_fields: bool) -> None: ...
    @property
    def scope(self) -> str: ...
    @scope.setter
    def scope(self, scope: str) -> None: ...
    @property
    def versions(self) -> list[V1CustomResourceDefinitionVersion]: ...
    @versions.setter
    def versions(self, versions: list[V1CustomResourceDefinitionVersion]) -> None: ...



from typing import Any

class V1ServerAddressByClientCIDR:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, client_cidr: str | None = None, server_address: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def client_cidr(self) -> str: ...
    @client_cidr.setter
    def client_cidr(self, client_cidr: str) -> None: ...
    @property
    def server_address(self) -> str: ...
    @server_address.setter
    def server_address(self, server_address: str) -> None: ...



from kubernetes_asyncio.client.models import V1VolumeNodeResources
from typing import Any

class V1CSINodeDriver:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, allocatable: V1VolumeNodeResources | None = None, name: str | None = None, node_id: str | None = None, topology_keys: list[str] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def allocatable(self) -> V1VolumeNodeResources: ...
    @allocatable.setter
    def allocatable(self, allocatable: V1VolumeNodeResources) -> None: ...
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, name: str) -> None: ...
    @property
    def node_id(self) -> str: ...
    @node_id.setter
    def node_id(self, node_id: str) -> None: ...
    @property
    def topology_keys(self) -> list[str]: ...
    @topology_keys.setter
    def topology_keys(self, topology_keys: list[str]) -> None: ...



from kubernetes_asyncio.client.models import V1QueuingConfiguration
from typing import Any

class V1LimitResponse:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, queuing: V1QueuingConfiguration | None = None, type: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def queuing(self) -> V1QueuingConfiguration: ...
    @queuing.setter
    def queuing(self, queuing: V1QueuingConfiguration) -> None: ...
    @property
    def type(self) -> str: ...
    @type.setter
    def type(self, type: str) -> None: ...



from kubernetes_asyncio.client.models import V2ContainerResourceMetricStatus
from kubernetes_asyncio.client.models import V2ExternalMetricStatus
from kubernetes_asyncio.client.models import V2ObjectMetricStatus
from kubernetes_asyncio.client.models import V2PodsMetricStatus
from kubernetes_asyncio.client.models import V2ResourceMetricStatus
from typing import Any

class V2MetricStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, container_resource: V2ContainerResourceMetricStatus | None = None, external: V2ExternalMetricStatus | None = None, object: V2ObjectMetricStatus | None = None, pods: V2PodsMetricStatus | None = None, resource: V2ResourceMetricStatus | None = None, type: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def container_resource(self) -> V2ContainerResourceMetricStatus: ...
    @container_resource.setter
    def container_resource(self, container_resource: V2ContainerResourceMetricStatus) -> None: ...
    @property
    def external(self) -> V2ExternalMetricStatus: ...
    @external.setter
    def external(self, external: V2ExternalMetricStatus) -> None: ...
    @property
    def object(self) -> V2ObjectMetricStatus: ...
    @object.setter
    def object(self, object: V2ObjectMetricStatus) -> None: ...
    @property
    def pods(self) -> V2PodsMetricStatus: ...
    @pods.setter
    def pods(self, pods: V2PodsMetricStatus) -> None: ...
    @property
    def resource(self) -> V2ResourceMetricStatus: ...
    @resource.setter
    def resource(self, resource: V2ResourceMetricStatus) -> None: ...
    @property
    def type(self) -> str: ...
    @type.setter
    def type(self, type: str) -> None: ...



from kubernetes_asyncio.client.models import V1ObjectMeta
from kubernetes_asyncio.client.models import V1beta1MutatingAdmissionPolicyBindingSpec
from typing import Any

class V1beta1MutatingAdmissionPolicyBinding:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, api_version: str | None = None, kind: str | None = None, metadata: V1ObjectMeta | None = None, spec: V1beta1MutatingAdmissionPolicyBindingSpec | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def api_version(self) -> str: ...
    @api_version.setter
    def api_version(self, api_version: str) -> None: ...
    @property
    def kind(self) -> str: ...
    @kind.setter
    def kind(self, kind: str) -> None: ...
    @property
    def metadata(self) -> V1ObjectMeta: ...
    @metadata.setter
    def metadata(self, metadata: V1ObjectMeta) -> None: ...
    @property
    def spec(self) -> V1beta1MutatingAdmissionPolicyBindingSpec: ...
    @spec.setter
    def spec(self, spec: V1beta1MutatingAdmissionPolicyBindingSpec) -> None: ...



from kubernetes_asyncio.client.models import V1SecretReference
from typing import Any

class V1ScaleIOPersistentVolumeSource:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, fs_type: str | None = None, gateway: str | None = None, protection_domain: str | None = None, read_only: bool | None = None, secret_ref: V1SecretReference | None = None, ssl_enabled: bool | None = None, storage_mode: str | None = None, storage_pool: str | None = None, system: str | None = None, volume_name: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def fs_type(self) -> str: ...
    @fs_type.setter
    def fs_type(self, fs_type: str) -> None: ...
    @property
    def gateway(self) -> str: ...
    @gateway.setter
    def gateway(self, gateway: str) -> None: ...
    @property
    def protection_domain(self) -> str: ...
    @protection_domain.setter
    def protection_domain(self, protection_domain: str) -> None: ...
    @property
    def read_only(self) -> bool: ...
    @read_only.setter
    def read_only(self, read_only: bool) -> None: ...
    @property
    def secret_ref(self) -> V1SecretReference: ...
    @secret_ref.setter
    def secret_ref(self, secret_ref: V1SecretReference) -> None: ...
    @property
    def ssl_enabled(self) -> bool: ...
    @ssl_enabled.setter
    def ssl_enabled(self, ssl_enabled: bool) -> None: ...
    @property
    def storage_mode(self) -> str: ...
    @storage_mode.setter
    def storage_mode(self, storage_mode: str) -> None: ...
    @property
    def storage_pool(self) -> str: ...
    @storage_pool.setter
    def storage_pool(self, storage_pool: str) -> None: ...
    @property
    def system(self) -> str: ...
    @system.setter
    def system(self, system: str) -> None: ...
    @property
    def volume_name(self) -> str: ...
    @volume_name.setter
    def volume_name(self, volume_name: str) -> None: ...



from typing import Any

class V1QueuingConfiguration:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, hand_size: int | None = None, queue_length_limit: int | None = None, queues: int | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def hand_size(self) -> int: ...
    @hand_size.setter
    def hand_size(self, hand_size: int) -> None: ...
    @property
    def queue_length_limit(self) -> int: ...
    @queue_length_limit.setter
    def queue_length_limit(self, queue_length_limit: int) -> None: ...
    @property
    def queues(self) -> int: ...
    @queues.setter
    def queues(self, queues: int) -> None: ...



from kubernetes_asyncio.client.models import V1LabelSelector
from kubernetes_asyncio.client.models import V1NamedRuleWithOperations
from typing import Any

class V1MatchResources:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, exclude_resource_rules: list[V1NamedRuleWithOperations] | None = None, match_policy: str | None = None, namespace_selector: V1LabelSelector | None = None, object_selector: V1LabelSelector | None = None, resource_rules: list[V1NamedRuleWithOperations] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def exclude_resource_rules(self) -> list[V1NamedRuleWithOperations]: ...
    @exclude_resource_rules.setter
    def exclude_resource_rules(self, exclude_resource_rules: list[V1NamedRuleWithOperations]) -> None: ...
    @property
    def match_policy(self) -> str: ...
    @match_policy.setter
    def match_policy(self, match_policy: str) -> None: ...
    @property
    def namespace_selector(self) -> V1LabelSelector: ...
    @namespace_selector.setter
    def namespace_selector(self, namespace_selector: V1LabelSelector) -> None: ...
    @property
    def object_selector(self) -> V1LabelSelector: ...
    @object_selector.setter
    def object_selector(self, object_selector: V1LabelSelector) -> None: ...
    @property
    def resource_rules(self) -> list[V1NamedRuleWithOperations]: ...
    @resource_rules.setter
    def resource_rules(self, resource_rules: list[V1NamedRuleWithOperations]) -> None: ...



from logging import FileHandler, Logger
from typing import Any, Callable, Coroutine
from kubernetes_asyncio.client.exceptions import ApiValueError as ApiValueError

JSON_SCHEMA_VALIDATION_KEYWORDS: set[str]

class Configuration:
    server_index: int
    server_operation_index: dict
    server_variables: dict
    server_operation_variables: dict
    temp_folder_path: str | None
    api_key: dict
    api_key_prefix: dict
    refresh_api_key_hook: Coroutine | Callable | None
    username: str | None
    password: str | None
    discard_unknown_keys: bool
    disabled_client_side_validations: str
    logger: dict[str, Logger]
    logger_stream_handler: Logger | None
    logger_file_handler: Logger | None
    verify_ssl: bool
    disable_strict_ssl_verification: bool
    ssl_ca_cert: str | None
    cert_file: str | None
    key_file: str | None
    assert_hostname: bool | None
    tls_server_name: str | None
    connection_pool_maxsize: int
    proxy: str | None
    proxy_headers: dict | None
    safe_chars_for_path_param: str
    retries: int | None
    client_side_validation: bool
    socket_options: dict | None
    def __init__(
        self,
        host: str | None = None,
        api_key: dict | None = None,
        api_key_prefix: dict | None = None,
        username: str | None = None,
        password: str | None = None,
        discard_unknown_keys: bool = False,
        disabled_client_side_validations: str = "",
        server_index: int | None = None,
        server_variables: dict | None = None,
        server_operation_index: dict | None = None,
        server_operation_variables: dict | None = None,
        ssl_ca_cert: str | None = None,
    ) -> None: ...
    def __deepcopy__(self, memo: Configuration): ...
    def __setattr__(self, name: str, value: Any) -> None: ...
    @classmethod
    def set_default(cls, default: Configuration | None) -> None: ...
    @classmethod
    def get_default(cls) -> Configuration: ...
    @classmethod
    def get_default_copy(cls) -> Configuration: ...
    @property
    def logger_file(self): ...
    @logger_file.setter
    def logger_file(self, value: str) -> FileHandler | None: ...
    @property
    def debug(self) -> bool: ...
    @debug.setter
    def debug(self, value: bool) -> None: ...
    @property
    def logger_format(self) -> str | None: ...
    @logger_format.setter
    def logger_format(self, value: str) -> None: ...
    async def get_api_key_with_prefix(
        self, identifier: str, alias: str | None = None
    ) -> str: ...
    def get_basic_auth_token(self) -> str: ...
    async def auth_settings(self) -> dict: ...
    def to_debug_report(self) -> str: ...
    def get_host_settings(self) -> list[dict[str, str]]: ...
    def get_host_from_settings(
        self,
        index: int,
        variables: dict | None = None,
        servers: dict | None = None,
    ) -> str | None: ...
    @property
    def host(self) -> str | None: ...
    @host.setter
    def host(self, value: str | None) -> None: ...



from kubernetes_asyncio.client.models import V1beta2DeviceClaimConfiguration
from kubernetes_asyncio.client.models import V1beta2DeviceConstraint
from kubernetes_asyncio.client.models import V1beta2DeviceRequest
from typing import Any

class V1beta2DeviceClaim:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, config: list[V1beta2DeviceClaimConfiguration] | None = None, constraints: list[V1beta2DeviceConstraint] | None = None, requests: list[V1beta2DeviceRequest] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def config(self) -> list[V1beta2DeviceClaimConfiguration]: ...
    @config.setter
    def config(self, config: list[V1beta2DeviceClaimConfiguration]) -> None: ...
    @property
    def constraints(self) -> list[V1beta2DeviceConstraint]: ...
    @constraints.setter
    def constraints(self, constraints: list[V1beta2DeviceConstraint]) -> None: ...
    @property
    def requests(self) -> list[V1beta2DeviceRequest]: ...
    @requests.setter
    def requests(self, requests: list[V1beta2DeviceRequest]) -> None: ...



from kubernetes_asyncio.client.models import V1HTTPIngressRuleValue
from typing import Any

class V1IngressRule:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, host: str | None = None, http: V1HTTPIngressRuleValue | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def host(self) -> str: ...
    @host.setter
    def host(self, host: str) -> None: ...
    @property
    def http(self) -> V1HTTPIngressRuleValue: ...
    @http.setter
    def http(self, http: V1HTTPIngressRuleValue) -> None: ...



from kubernetes_asyncio.client.models import V1beta1OpaqueDeviceConfiguration
from typing import Any

class V1beta1DeviceAllocationConfiguration:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, opaque: V1beta1OpaqueDeviceConfiguration | None = None, requests: list[str] | None = None, source: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def opaque(self) -> V1beta1OpaqueDeviceConfiguration: ...
    @opaque.setter
    def opaque(self, opaque: V1beta1OpaqueDeviceConfiguration) -> None: ...
    @property
    def requests(self) -> list[str]: ...
    @requests.setter
    def requests(self, requests: list[str]) -> None: ...
    @property
    def source(self) -> str: ...
    @source.setter
    def source(self, source: str) -> None: ...



from datetime import datetime
from kubernetes_asyncio.client.models import V1ObjectReference
from typing import Any

class V1CronJobStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, active: list[V1ObjectReference] | None = None, last_schedule_time: datetime | None = None, last_successful_time: datetime | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def active(self) -> list[V1ObjectReference]: ...
    @active.setter
    def active(self, active: list[V1ObjectReference]) -> None: ...
    @property
    def last_schedule_time(self) -> datetime: ...
    @last_schedule_time.setter
    def last_schedule_time(self, last_schedule_time: datetime) -> None: ...
    @property
    def last_successful_time(self) -> datetime: ...
    @last_successful_time.setter
    def last_successful_time(self, last_successful_time: datetime) -> None: ...



from kubernetes_asyncio.client.api_client import ApiClient
from kubernetes_asyncio.client.models import V1APIResourceList
from kubernetes_asyncio.client.models import V1FlowSchema
from kubernetes_asyncio.client.models import V1FlowSchemaList
from kubernetes_asyncio.client.models import V1PriorityLevelConfiguration
from kubernetes_asyncio.client.models import V1PriorityLevelConfigurationList
from kubernetes_asyncio.client.models import V1Status
from multidict import CIMultiDictProxy
from typing import Any
from typing import Awaitable

class FlowcontrolApiserverV1Api:
    def __init__(self, api_client: ApiClient | None = None) -> None: ...
    def create_flow_schema(self, body: V1FlowSchema, **kwargs: Any) -> Awaitable[V1FlowSchema]: ...
    def create_flow_schema_with_http_info(self, body: V1FlowSchema, **kwargs: Any) -> Awaitable[tuple[V1FlowSchema, int, CIMultiDictProxy]]: ...
    def create_priority_level_configuration(self, body: V1PriorityLevelConfiguration, **kwargs: Any) -> Awaitable[V1PriorityLevelConfiguration]: ...
    def create_priority_level_configuration_with_http_info(self, body: V1PriorityLevelConfiguration, **kwargs: Any) -> Awaitable[tuple[V1PriorityLevelConfiguration, int, CIMultiDictProxy]]: ...
    def delete_collection_flow_schema(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_flow_schema_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_priority_level_configuration(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_priority_level_configuration_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_flow_schema(self, name: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_flow_schema_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_priority_level_configuration(self, name: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_priority_level_configuration_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def get_api_resources(self, **kwargs: Any) -> Awaitable[V1APIResourceList]: ...
    def get_api_resources_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1APIResourceList, int, CIMultiDictProxy]]: ...
    def list_flow_schema(self, **kwargs: Any) -> Awaitable[V1FlowSchemaList]: ...
    def list_flow_schema_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1FlowSchemaList, int, CIMultiDictProxy]]: ...
    def list_priority_level_configuration(self, **kwargs: Any) -> Awaitable[V1PriorityLevelConfigurationList]: ...
    def list_priority_level_configuration_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1PriorityLevelConfigurationList, int, CIMultiDictProxy]]: ...
    def patch_flow_schema(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1FlowSchema]: ...
    def patch_flow_schema_status(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1FlowSchema]: ...
    def patch_flow_schema_status_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1FlowSchema, int, CIMultiDictProxy]]: ...
    def patch_flow_schema_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1FlowSchema, int, CIMultiDictProxy]]: ...
    def patch_priority_level_configuration(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1PriorityLevelConfiguration]: ...
    def patch_priority_level_configuration_status(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1PriorityLevelConfiguration]: ...
    def patch_priority_level_configuration_status_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1PriorityLevelConfiguration, int, CIMultiDictProxy]]: ...
    def patch_priority_level_configuration_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1PriorityLevelConfiguration, int, CIMultiDictProxy]]: ...
    def read_flow_schema(self, name: str, **kwargs: Any) -> Awaitable[V1FlowSchema]: ...
    def read_flow_schema_status(self, name: str, **kwargs: Any) -> Awaitable[V1FlowSchema]: ...
    def read_flow_schema_status_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1FlowSchema, int, CIMultiDictProxy]]: ...
    def read_flow_schema_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1FlowSchema, int, CIMultiDictProxy]]: ...
    def read_priority_level_configuration(self, name: str, **kwargs: Any) -> Awaitable[V1PriorityLevelConfiguration]: ...
    def read_priority_level_configuration_status(self, name: str, **kwargs: Any) -> Awaitable[V1PriorityLevelConfiguration]: ...
    def read_priority_level_configuration_status_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1PriorityLevelConfiguration, int, CIMultiDictProxy]]: ...
    def read_priority_level_configuration_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1PriorityLevelConfiguration, int, CIMultiDictProxy]]: ...
    def replace_flow_schema(self, name: str, body: V1FlowSchema, **kwargs: Any) -> Awaitable[V1FlowSchema]: ...
    def replace_flow_schema_status(self, name: str, body: V1FlowSchema, **kwargs: Any) -> Awaitable[V1FlowSchema]: ...
    def replace_flow_schema_status_with_http_info(self, name: str, body: V1FlowSchema, **kwargs: Any) -> Awaitable[tuple[V1FlowSchema, int, CIMultiDictProxy]]: ...
    def replace_flow_schema_with_http_info(self, name: str, body: V1FlowSchema, **kwargs: Any) -> Awaitable[tuple[V1FlowSchema, int, CIMultiDictProxy]]: ...
    def replace_priority_level_configuration(self, name: str, body: V1PriorityLevelConfiguration, **kwargs: Any) -> Awaitable[V1PriorityLevelConfiguration]: ...
    def replace_priority_level_configuration_status(self, name: str, body: V1PriorityLevelConfiguration, **kwargs: Any) -> Awaitable[V1PriorityLevelConfiguration]: ...
    def replace_priority_level_configuration_status_with_http_info(self, name: str, body: V1PriorityLevelConfiguration, **kwargs: Any) -> Awaitable[tuple[V1PriorityLevelConfiguration, int, CIMultiDictProxy]]: ...
    def replace_priority_level_configuration_with_http_info(self, name: str, body: V1PriorityLevelConfiguration, **kwargs: Any) -> Awaitable[tuple[V1PriorityLevelConfiguration, int, CIMultiDictProxy]]: ...



# Copyright 2019 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Any, Type

from kubernetes_asyncio import watch
from kubernetes_asyncio.client.api_client import ApiClient
from kubernetes_asyncio.client.rest import ApiException
from kubernetes_asyncio.dynamic.discovery import (
    Discoverer, EagerDiscoverer, LazyDiscoverer,
)
from kubernetes_asyncio.dynamic.exceptions import (
    KubernetesValidateMissing, api_exception,
)
from kubernetes_asyncio.dynamic.resource import (
    Resource, ResourceField, ResourceInstance, ResourceList, Subresource,
)
from kubernetes_asyncio.watch.watch import Watch

try:
    import kubernetes_validate

    HAS_KUBERNETES_VALIDATE = True
except ImportError:
    HAS_KUBERNETES_VALIDATE = False

try:
    from kubernetes_validate.utils import VersionNotSupportedError
except ImportError:

    class VersionNotSupportedError(NotImplementedError):
        pass


__all__ = [
    "DynamicClient",
    "ResourceInstance",
    "Resource",
    "ResourceList",
    "Subresource",
    "EagerDiscoverer",
    "LazyDiscoverer",
    "ResourceField",
]


def meta_request(func):
    """Handles parsing response structure and translating API Exceptions"""

    async def inner(self, *args, **kwargs):
        serialize_response = kwargs.pop("serialize", True)
        serializer = kwargs.pop("serializer", ResourceInstance)
        try:
            resp = await func(self, *args, **kwargs)
        except ApiException as e:
            raise api_exception(e)
        if serialize_response:
            try:
                data = await resp.json()
                return serializer(self, data)
            except ValueError:
                data = await resp.json()
                return data
        return resp

    return inner


class DynamicClient(object):
    """A kubernetes client that dynamically discovers and interacts with
    the kubernetes API
    """

    def __init__(
        self,
        client: ApiClient,
        cache_file: str | None = None,
        discoverer: Type[Discoverer] | None = None,
    ) -> None:
        self.cache_file = cache_file
        self.client = client
        self.configuration = client.configuration
        self.discoverer: Type[Discoverer] = discoverer or LazyDiscoverer

    def __await__(self):
        async def closure():
            self.__discoverer = await self.discoverer(self, self.cache_file)
            return self

        return closure().__await__()

    async def __aenter__(self) -> "DynamicClient":
        self.__discoverer = await self.discoverer(self, self.cache_file)
        return self

    async def __aexit__(self, *args, **kwargs) -> None:
        return

    @property
    def resources(self) -> Discoverer:
        return self.__discoverer

    @property
    def version(self) -> dict[str, Any]:
        return self.__discoverer.version  # type: ignore

    @staticmethod
    def ensure_namespace(resource: Resource, namespace: str, body: Any) -> str:
        namespace = namespace or body.get("metadata", {}).get("namespace")
        if not namespace:
            raise ValueError(
                "Namespace is required for {}.{}".format(
                    resource.group_version, resource.kind
                )
            )
        return namespace

    @staticmethod
    def serialize_body(body: ResourceInstance | ResourceField | dict) -> dict:
        """Serialize body to raw dict so apiserver can handle it

        :param body: kubernetes resource body, current support: Union[Dict, ResourceInstance]
        """
        # This should match any `ResourceInstance` instances
        if callable(getattr(body, "to_dict", None)):
            return body.to_dict()  # type: ignore
        return body or {}  # type: ignore

    async def get(
        self,
        resource: Resource,
        name: str | None = None,
        namespace: str | None = None,
        **kwargs: Any,
    ) -> Any:
        path = resource.path(name=name, namespace=namespace)
        return await self.request("get", path, **kwargs)

    async def create(
        self,
        resource: Resource,
        body: dict | ResourceInstance | None = None,
        namespace: str | None = None,
        **kwargs,
    ) -> Any:
        if body is None:
            body = {}

        body = self.serialize_body(body)
        if resource.namespaced:
            if not namespace:
                raise ValueError("namespace is required")
            namespace = self.ensure_namespace(resource, namespace, body)
        path = resource.path(namespace=namespace)
        return await self.request("post", path, body=body, **kwargs)

    async def delete(
        self,
        resource: Resource,
        name: str | None = None,
        namespace: str | None = None,
        body: dict | ResourceInstance | None = None,
        label_selector: str | None = None,
        field_selector: str | None = None,
        **kwargs: Any,
    ) -> Any:
        if not (name or label_selector or field_selector):
            raise ValueError(
                "At least one of name|label_selector|field_selector is required"
            )
        if resource.namespaced and not (label_selector or field_selector or namespace):
            raise ValueError(
                "At least one of namespace|label_selector|field_selector is required"
            )
        path = resource.path(name=name, namespace=namespace)
        return await self.request(
            "delete",
            path,
            body=body,
            label_selector=label_selector,
            field_selector=field_selector,
            **kwargs,
        )

    async def replace(
        self,
        resource: Resource,
        body: dict | ResourceInstance | None = None,
        name: str | None = None,
        namespace: str | None = None,
        **kwargs: Any,
    ) -> Any:
        if body is None:
            body = {}

        body = self.serialize_body(body)
        name = name or body.get("metadata", {}).get("name")
        if not name:
            raise ValueError(
                "name is required to replace {}.{}".format(
                    resource.group_version, resource.kind
                )
            )
        if resource.namespaced:
            if not namespace:
                raise ValueError("namespace is required")
            namespace = self.ensure_namespace(resource, namespace, body)
        path = resource.path(name=name, namespace=namespace)
        return await self.request("put", path, body=body, **kwargs)

    async def patch(
        self,
        resource: Resource,
        body: dict | ResourceInstance | None = None,
        name: str | None = None,
        namespace: str | None = None,
        **kwargs: Any,
    ) -> Any:
        if body is None:
            body = {}
        body = self.serialize_body(body)
        name = name or body.get("metadata", {}).get("name")
        if not name:
            raise ValueError(
                "name is required to patch {}.{}".format(
                    resource.group_version, resource.kind
                )
            )
        if resource.namespaced:
            if not namespace:
                raise ValueError("namespace is required")
            namespace = self.ensure_namespace(resource, namespace, body)

        content_type = kwargs.pop(
            "content_type", "application/strategic-merge-patch+json"
        )
        path = resource.path(name=name, namespace=namespace)

        return await self.request(
            "patch", path, body=body, content_type=content_type, **kwargs
        )

    async def server_side_apply(
        self,
        resource: Resource,
        body: dict | ResourceInstance | None = None,
        name: str | None = None,
        namespace: str | None = None,
        force_conflicts: bool | None = None,
        **kwargs: Any,
    ) -> Any:
        if body is None:
            body = {}

        body = self.serialize_body(body)
        name = name or body.get("metadata", {}).get("name")
        if not name:
            raise ValueError(
                "name is required to patch {}.{}".format(
                    resource.group_version, resource.kind
                )
            )
        if resource.namespaced:
            if not namespace:
                raise ValueError("namespace is required")
            namespace = self.ensure_namespace(resource, namespace, body)

        # force content type to 'application/apply-patch+yaml'
        kwargs.update({"content_type": "application/apply-patch+yaml"})
        path = resource.path(name=name, namespace=namespace)

        return await self.request(
            "patch", path, body=body, force_conflicts=force_conflicts, **kwargs
        )

    @staticmethod
    async def watch(
        resource: Resource,
        namespace: str | None = None,
        name: str | None = None,
        label_selector: str | None = None,
        field_selector: str | None = None,
        resource_version: int | None = None,
        timeout: int | None = None,
        watcher: Watch | None = None,
    ) -> Any:
        """
        Stream events for a resource from the Kubernetes API

        :param resource: The API resource object that will be used to query the API
        :param namespace: The namespace to query
        :param name: The name of the resource instance to query
        :param label_selector: The label selector with which to filter results
        :param field_selector: The field selector with which to filter results
        :param resource_version: The version with which to filter results. Only events with
                                 a resource_version greater than this value will be returned
        :param timeout: The amount of time in seconds to wait before terminating the stream
        :param watcher: The Watcher object that will be used to stream the resource

        :return: Event object with these keys:
                   'type': The type of event such as "ADDED", "DELETED", etc.
                   'raw_object': a dict representing the watched object.
                   'object': A ResourceInstance wrapping raw_object.

        Example:
            client = DynamicClient(k8s_client)
            watcher = watch.Watch()
            v1_pods = client.resources.get(api_version='v1', kind='Pod')

            for e in v1_pods.watch(resource_version=0, namespace=default, timeout=5, watcher=watcher):
                print(e['type'])
                print(e['object'].metadata)
                # If you want to gracefully stop the stream watcher
                watcher.stop()
        """
        if not watcher:
            watcher = watch.Watch()

        # Use field selector to query for named instance so the watch parameter is handled properly.
        if name:
            field_selector = f"metadata.name={name}"

        kwargs = {}
        if timeout is not None:
            kwargs["timeout_seconds"] = timeout

        async for event in watcher.stream(
            resource.get,
            namespace=namespace,
            field_selector=field_selector,
            label_selector=label_selector,
            resource_version=resource_version,
            serialize=False,
            **kwargs,
        ):
            if event == "" or event is None:
                break
            event["object"] = ResourceInstance(resource, event["object"])
            yield event

    @meta_request
    async def request(self, method: str, path: str, body=None, **params) -> Any:
        if not path.startswith("/"):
            path = "/" + path

        path_params = params.get("path_params", {})
        query_params = params.get("query_params", [])
        if params.get("pretty") is not None:
            query_params.append(("pretty", params["pretty"]))
        if params.get("_continue") is not None:
            query_params.append(("continue", params["_continue"]))
        if params.get("include_uninitialized") is not None:
            query_params.append(
                ("includeUninitialized", params["include_uninitialized"])
            )
        if params.get("field_selector") is not None:
            query_params.append(("fieldSelector", params["field_selector"]))
        if params.get("label_selector") is not None:
            query_params.append(("labelSelector", params["label_selector"]))
        if params.get("limit") is not None:
            query_params.append(("limit", params["limit"]))
        if params.get("resource_version") is not None:
            query_params.append(("resourceVersion", params["resource_version"]))
        if params.get("timeout_seconds") is not None:
            query_params.append(("timeoutSeconds", params["timeout_seconds"]))
        if params.get("watch") is not None:
            query_params.append(("watch", params["watch"]))
        if params.get("grace_period_seconds") is not None:
            query_params.append(("gracePeriodSeconds", params["grace_period_seconds"]))
        if params.get("propagation_policy") is not None:
            query_params.append(("propagationPolicy", params["propagation_policy"]))
        if params.get("orphan_dependents") is not None:
            query_params.append(("orphanDependents", params["orphan_dependents"]))
        if params.get("dry_run") is not None:
            query_params.append(("dryRun", params["dry_run"]))
        if params.get("field_manager") is not None:
            query_params.append(("fieldManager", params["field_manager"]))
        if params.get("force_conflicts") is not None:
            query_params.append(("force", params["force_conflicts"]))

        header_params = params.get("header_params", {})

        # Checking Accept header.
        new_header_params = dict(
            (key.lower(), value) for key, value in header_params.items()
        )
        if "accept" not in new_header_params:
            header_params["Accept"] = self.client.select_header_accept(
                [
                    "application/json",
                    "application/yaml",
                ]
            )

        # HTTP header `Content-Type`
        if params.get("content_type"):
            header_params["Content-Type"] = params["content_type"]
        else:
            header_params["Content-Type"] = self.client.select_header_content_type(
                ["*/*"]
            )

        # Authentication setting
        auth_settings = ["BearerToken"]

        api_response = await self.client.call_api(
            path,
            method.upper(),
            path_params,
            query_params,
            header_params,
            body=body,
            async_req=params.get("async_req"),
            auth_settings=auth_settings,
            _preload_content=False,
            _return_http_data_only=params.get("_return_http_data_only", True),
            _request_timeout=params.get("_request_timeout"),
        )
        if params.get("async_req"):
            return api_response.get()
        else:
            return api_response

    def validate(
        self, definition: dict, version: str | None = None, strict: bool = False
    ):
        """validate checks a kubernetes resource definition

        Args:
            definition (dict): resource definition
            version (str): version of kubernetes to validate against
            strict (bool): whether unexpected additional properties should be considered errors

        Returns:
            warnings (list), errors (list): warnings are missing validations, errors are validation failures
        """
        if not HAS_KUBERNETES_VALIDATE:
            raise KubernetesValidateMissing()

        errors = list()
        warnings = list()
        try:
            if version is None:
                try:
                    version = self.version["kubernetes"]["gitVersion"]
                except KeyError:
                    version = kubernetes_validate.latest_version()
            assert version
            kubernetes_validate.validate(definition, version, strict)
        except kubernetes_validate.utils.ValidationError as e:
            errors.append(
                "resource definition validation error at %s: %s"
                % (".".join([str(item) for item in e.path]), e.message)
            )  # noqa: B306
        except VersionNotSupportedError:
            errors.append(
                "Kubernetes version %s is not supported by kubernetes-validate"
                % version
            )
        except kubernetes_validate.utils.SchemaNotFoundError as e:
            warnings.append(
                "Could not find schema for object kind %s with API version %s in Kubernetes version %s"
                " (possibly Custom Resource?)" % (e.kind, e.api_version, e.version)
            )
        return warnings, errors



# Copyright 2021 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import logging
from typing import Any

from kubernetes_asyncio import client
from kubernetes_asyncio.client.api_client import ApiClient
from kubernetes_asyncio.client.rest import ApiException
from kubernetes_asyncio.leaderelection.leaderelectionrecord import (
    LeaderElectionRecord,
)
from kubernetes_asyncio.leaderelection.resourcelock.baselock import BaseLock


class ConfigMapLock(BaseLock):
    def __init__(self, name: str, namespace: str, identity: str, api_client: ApiClient):
        """
        :param name: name of the lock
        :param namespace: namespace
        :param identity: A unique identifier that the candidate is using
        """
        super().__init__(name, namespace, identity)

        # self._api_instance = None # See api_instance property
        self.api_instance = client.CoreV1Api(api_client=api_client)
        self.leader_electionrecord_annotationkey = (
            "control-plane.alpha.kubernetes.io/leader"
        )
        self.configmap_reference: client.V1ConfigMap | None = None
        self.lock_record: dict[str, Any] = {
            "holderIdentity": None,
            "leaseDurationSeconds": None,
            "acquireTime": None,
            "renewTime": None,
        }

    # get returns the election record from a ConfigMap Annotation
    async def get(
        self, name: str, namespace: str
    ) -> tuple[bool, LeaderElectionRecord] | tuple[bool, Exception] | tuple[bool, None]:
        """
        :param name: Name of the configmap object information to get
        :param namespace: Namespace in which the configmap object is to be searched
        :return: 'True, election record' if object found else 'False, exception response'
        """
        try:
            api_response = await self.api_instance.read_namespaced_config_map(
                name, namespace
            )

            # If an annotation does not exist - add the leader_electionrecord_annotationkey
            annotations = api_response.metadata.annotations
            if annotations is None or annotations == "":
                api_response.metadata.annotations = {
                    self.leader_electionrecord_annotationkey: ""
                }
                self.configmap_reference = api_response
                return True, None

            # If an annotation exists but, the leader_electionrecord_annotationkey does not then add it as a key
            if not annotations.get(self.leader_electionrecord_annotationkey):
                api_response.metadata.annotations = {
                    self.leader_electionrecord_annotationkey: ""
                }
                self.configmap_reference = api_response
                return True, None

            lock_record = self.get_lock_object(
                json.loads(annotations[self.leader_electionrecord_annotationkey])
            )

            self.configmap_reference = api_response
            return True, lock_record
        except ApiException as e:
            return False, e

    async def create(
        self, name: str, namespace: str, election_record: LeaderElectionRecord
    ) -> bool:
        """
        :param electionRecord: Annotation string
        :param name: Name of the configmap object to be created
        :param namespace: Namespace in which the configmap object is to be created
        :return: 'True' if object is created else 'False' if failed
        """
        body = client.V1ConfigMap(
            metadata=client.V1ObjectMeta(
                name=name,
                annotations={
                    self.leader_electionrecord_annotationkey: json.dumps(
                        self.get_lock_dict(election_record)
                    )
                },
            )
        )

        try:
            await self.api_instance.create_namespaced_config_map(
                namespace, body, pretty=True
            )
            return True
        except ApiException:
            logging.exception("Failed to create lock")
            return False

    async def update(
        self, name: str, namespace: str, updated_record: LeaderElectionRecord
    ) -> bool:
        """
        :param name: name of the lock to be updated
        :param namespace: namespace the lock is in
        :param updated_record: the updated election record
        :return: True if update is successful False if it fails
        """
        try:
            # Set the updated record
            assert self.configmap_reference is not None
            self.configmap_reference.metadata.annotations[
                self.leader_electionrecord_annotationkey
            ] = json.dumps(self.get_lock_dict(updated_record))
            await self.api_instance.replace_namespaced_config_map(
                name=name, namespace=namespace, body=self.configmap_reference
            )
            return True
        except ApiException:
            logging.exception("Failed to update lock")
            return False

    def get_lock_object(self, lock_record: dict) -> LeaderElectionRecord:
        leader_election_record = LeaderElectionRecord(None, None, None, None)

        if lock_record.get("holderIdentity"):
            leader_election_record.holder_identity = lock_record["holderIdentity"]
        if lock_record.get("leaseDurationSeconds"):
            leader_election_record.lease_duration = lock_record["leaseDurationSeconds"]
        if lock_record.get("acquireTime"):
            leader_election_record.acquire_time = lock_record["acquireTime"]
        if lock_record.get("renewTime"):
            leader_election_record.renew_time = lock_record["renewTime"]

        return leader_election_record

    def get_lock_dict(
        self, leader_election_record: LeaderElectionRecord
    ) -> dict[str, Any]:
        self.lock_record["holderIdentity"] = leader_election_record.holder_identity
        self.lock_record["leaseDurationSeconds"] = leader_election_record.lease_duration
        self.lock_record["acquireTime"] = leader_election_record.acquire_time
        self.lock_record["renewTime"] = leader_election_record.renew_time

        return self.lock_record



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1.34.3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from kubernetes_asyncio.client.configuration import Configuration


class V1VolumeMountStatus(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'mount_path': 'str',
        'name': 'str',
        'read_only': 'bool',
        'recursive_read_only': 'str'
    }

    attribute_map = {
        'mount_path': 'mountPath',
        'name': 'name',
        'read_only': 'readOnly',
        'recursive_read_only': 'recursiveReadOnly'
    }

    def __init__(self, mount_path=None, name=None, read_only=None, recursive_read_only=None, local_vars_configuration=None):  # noqa: E501
        """V1VolumeMountStatus - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default()
        self.local_vars_configuration = local_vars_configuration

        self._mount_path = None
        self._name = None
        self._read_only = None
        self._recursive_read_only = None
        self.discriminator = None

        self.mount_path = mount_path
        self.name = name
        if read_only is not None:
            self.read_only = read_only
        if recursive_read_only is not None:
            self.recursive_read_only = recursive_read_only

    @property
    def mount_path(self):
        """Gets the mount_path of this V1VolumeMountStatus.  # noqa: E501

        MountPath corresponds to the original VolumeMount.  # noqa: E501

        :return: The mount_path of this V1VolumeMountStatus.  # noqa: E501
        :rtype: str
        """
        return self._mount_path

    @mount_path.setter
    def mount_path(self, mount_path):
        """Sets the mount_path of this V1VolumeMountStatus.

        MountPath corresponds to the original VolumeMount.  # noqa: E501

        :param mount_path: The mount_path of this V1VolumeMountStatus.  # noqa: E501
        :type mount_path: str
        """
        if self.local_vars_configuration.client_side_validation and mount_path is None:  # noqa: E501
            raise ValueError("Invalid value for `mount_path`, must not be `None`")  # noqa: E501

        self._mount_path = mount_path

    @property
    def name(self):
        """Gets the name of this V1VolumeMountStatus.  # noqa: E501

        Name corresponds to the name of the original VolumeMount.  # noqa: E501

        :return: The name of this V1VolumeMountStatus.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this V1VolumeMountStatus.

        Name corresponds to the name of the original VolumeMount.  # noqa: E501

        :param name: The name of this V1VolumeMountStatus.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def read_only(self):
        """Gets the read_only of this V1VolumeMountStatus.  # noqa: E501

        ReadOnly corresponds to the original VolumeMount.  # noqa: E501

        :return: The read_only of this V1VolumeMountStatus.  # noqa: E501
        :rtype: bool
        """
        return self._read_only

    @read_only.setter
    def read_only(self, read_only):
        """Sets the read_only of this V1VolumeMountStatus.

        ReadOnly corresponds to the original VolumeMount.  # noqa: E501

        :param read_only: The read_only of this V1VolumeMountStatus.  # noqa: E501
        :type read_only: bool
        """

        self._read_only = read_only

    @property
    def recursive_read_only(self):
        """Gets the recursive_read_only of this V1VolumeMountStatus.  # noqa: E501

        RecursiveReadOnly must be set to Disabled, Enabled, or unspecified (for non-readonly mounts). An IfPossible value in the original VolumeMount must be translated to Disabled or Enabled, depending on the mount result.  # noqa: E501

        :return: The recursive_read_only of this V1VolumeMountStatus.  # noqa: E501
        :rtype: str
        """
        return self._recursive_read_only

    @recursive_read_only.setter
    def recursive_read_only(self, recursive_read_only):
        """Sets the recursive_read_only of this V1VolumeMountStatus.

        RecursiveReadOnly must be set to Disabled, Enabled, or unspecified (for non-readonly mounts). An IfPossible value in the original VolumeMount must be translated to Disabled or Enabled, depending on the mount result.  # noqa: E501

        :param recursive_read_only: The recursive_read_only of this V1VolumeMountStatus.  # noqa: E501
        :type recursive_read_only: str
        """

        self._recursive_read_only = recursive_read_only

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1VolumeMountStatus):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1VolumeMountStatus):
            return True

        return self.to_dict() != other.to_dict()



# Copyright 2018 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import asyncio
import asyncio.subprocess
import json
import os
import sys
from typing import TYPE_CHECKING, Any, cast

if TYPE_CHECKING:
    from kubernetes_asyncio.config.kube_config import ConfigNode
from kubernetes_asyncio.config.config_exception import ConfigException


class ExecProvider:
    """
    Implementation of the proposal for out-of-tree client authentication providers
    as described here --
    https://github.com/kubernetes/community/blob/master/contributors/design-proposals/auth/kubectl-exec-plugins.md

    Missing from implementation:

    * TLS cert support
    * caching
    """

    def __init__(self, exec_config: "ConfigNode") -> None:
        for key in ["command", "apiVersion"]:
            if key not in exec_config:
                raise ConfigException(f"exec: malformed request. missing key '{key}'")
        self.api_version = exec_config["apiVersion"]
        self.args = [str(exec_config["command"])]
        if exec_config.safe_get("args"):
            ec_args = exec_config["args"]
            if not ec_args or not isinstance(ec_args.value, list):
                raise ConfigException(
                    f"exec: malformed request. invalid args list '{ec_args}'"
                )
            self.args.extend(ec_args.value)
        self.env = os.environ.copy()
        if exec_config.safe_get("env"):
            additional_vars = {}
            for item in cast(list, exec_config["env"]):
                name = item["name"]
                value = item["value"]
                additional_vars[name] = value
            self.env.update(additional_vars)

    async def run(self, previous_response: str | None = None) -> Any:
        # Validate the run can be executed on Windows
        if type(asyncio.get_event_loop()).__name__ == "_WindowsSelectorEventLoop":
            raise ConfigException(
                "exec: _WindowsSelectorEventLoop does NOT support subprocesses, see README.md"
            )

        kubernetes_exec_info: dict[str, Any] = {
            "apiVersion": self.api_version,
            "kind": "ExecCredential",
            "spec": {"interactive": sys.stdout.isatty()},
        }
        if previous_response:
            kubernetes_exec_info["spec"]["response"] = previous_response
        self.env["KUBERNETES_EXEC_INFO"] = json.dumps(kubernetes_exec_info)

        cmd_exec = asyncio.create_subprocess_exec(
            *self.args,
            env=self.env,
            stdin=None,
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
        )
        proc = await cmd_exec

        if proc.stdout:
            stdout = await proc.stdout.read()
        else:
            raise RuntimeError("Unable to read stdout")
        if proc.stderr:
            stderr = await proc.stderr.read()
        else:
            raise RuntimeError("Unable to read stderr")
        exit_code = await proc.wait()

        if exit_code != 0:
            msg = "exec: process returned %d" % exit_code
            stderr = stderr.strip()
            if stderr:
                msg += ". %s" % stderr.decode()
            raise ConfigException(msg)
        try:
            data = json.loads(stdout)
        except ValueError as de:
            raise ConfigException("exec: failed to decode process output: %s" % de)
        for key in ("apiVersion", "kind", "status"):
            if key not in data:
                raise ConfigException(
                    "exec: malformed response. missing key '%s'" % key
                )
        if data["apiVersion"] != self.api_version:
            raise ConfigException(
                "exec: plugin api version %s does not match %s"
                % (data["apiVersion"], self.api_version)
            )
        return data["status"]



from typing import Any

class V1ContainerImage:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, names: list[str] | None = None, size_bytes: int | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def names(self) -> list[str]: ...
    @names.setter
    def names(self, names: list[str]) -> None: ...
    @property
    def size_bytes(self) -> int: ...
    @size_bytes.setter
    def size_bytes(self, size_bytes: int) -> None: ...



from kubernetes_asyncio.client.models import V1alpha1ApplyConfiguration
from kubernetes_asyncio.client.models import V1alpha1JSONPatch
from typing import Any

class V1alpha1Mutation:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, apply_configuration: V1alpha1ApplyConfiguration | None = None, json_patch: V1alpha1JSONPatch | None = None, patch_type: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def apply_configuration(self) -> V1alpha1ApplyConfiguration: ...
    @apply_configuration.setter
    def apply_configuration(self, apply_configuration: V1alpha1ApplyConfiguration) -> None: ...
    @property
    def json_patch(self) -> V1alpha1JSONPatch: ...
    @json_patch.setter
    def json_patch(self, json_patch: V1alpha1JSONPatch) -> None: ...
    @property
    def patch_type(self) -> str: ...
    @patch_type.setter
    def patch_type(self, patch_type: str) -> None: ...



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1.34.3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from kubernetes_asyncio.client.configuration import Configuration


class V1ServiceAccount(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'api_version': 'str',
        'automount_service_account_token': 'bool',
        'image_pull_secrets': 'list[V1LocalObjectReference]',
        'kind': 'str',
        'metadata': 'V1ObjectMeta',
        'secrets': 'list[V1ObjectReference]'
    }

    attribute_map = {
        'api_version': 'apiVersion',
        'automount_service_account_token': 'automountServiceAccountToken',
        'image_pull_secrets': 'imagePullSecrets',
        'kind': 'kind',
        'metadata': 'metadata',
        'secrets': 'secrets'
    }

    def __init__(self, api_version=None, automount_service_account_token=None, image_pull_secrets=None, kind=None, metadata=None, secrets=None, local_vars_configuration=None):  # noqa: E501
        """V1ServiceAccount - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default()
        self.local_vars_configuration = local_vars_configuration

        self._api_version = None
        self._automount_service_account_token = None
        self._image_pull_secrets = None
        self._kind = None
        self._metadata = None
        self._secrets = None
        self.discriminator = None

        if api_version is not None:
            self.api_version = api_version
        if automount_service_account_token is not None:
            self.automount_service_account_token = automount_service_account_token
        if image_pull_secrets is not None:
            self.image_pull_secrets = image_pull_secrets
        if kind is not None:
            self.kind = kind
        if metadata is not None:
            self.metadata = metadata
        if secrets is not None:
            self.secrets = secrets

    @property
    def api_version(self):
        """Gets the api_version of this V1ServiceAccount.  # noqa: E501

        APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa: E501

        :return: The api_version of this V1ServiceAccount.  # noqa: E501
        :rtype: str
        """
        return self._api_version

    @api_version.setter
    def api_version(self, api_version):
        """Sets the api_version of this V1ServiceAccount.

        APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources  # noqa: E501

        :param api_version: The api_version of this V1ServiceAccount.  # noqa: E501
        :type api_version: str
        """

        self._api_version = api_version

    @property
    def automount_service_account_token(self):
        """Gets the automount_service_account_token of this V1ServiceAccount.  # noqa: E501

        AutomountServiceAccountToken indicates whether pods running as this service account should have an API token automatically mounted. Can be overridden at the pod level.  # noqa: E501

        :return: The automount_service_account_token of this V1ServiceAccount.  # noqa: E501
        :rtype: bool
        """
        return self._automount_service_account_token

    @automount_service_account_token.setter
    def automount_service_account_token(self, automount_service_account_token):
        """Sets the automount_service_account_token of this V1ServiceAccount.

        AutomountServiceAccountToken indicates whether pods running as this service account should have an API token automatically mounted. Can be overridden at the pod level.  # noqa: E501

        :param automount_service_account_token: The automount_service_account_token of this V1ServiceAccount.  # noqa: E501
        :type automount_service_account_token: bool
        """

        self._automount_service_account_token = automount_service_account_token

    @property
    def image_pull_secrets(self):
        """Gets the image_pull_secrets of this V1ServiceAccount.  # noqa: E501

        ImagePullSecrets is a list of references to secrets in the same namespace to use for pulling any images in pods that reference this ServiceAccount. ImagePullSecrets are distinct from Secrets because Secrets can be mounted in the pod, but ImagePullSecrets are only accessed by the kubelet. More info: https://kubernetes.io/docs/concepts/containers/images/#specifying-imagepullsecrets-on-a-pod  # noqa: E501

        :return: The image_pull_secrets of this V1ServiceAccount.  # noqa: E501
        :rtype: list[V1LocalObjectReference]
        """
        return self._image_pull_secrets

    @image_pull_secrets.setter
    def image_pull_secrets(self, image_pull_secrets):
        """Sets the image_pull_secrets of this V1ServiceAccount.

        ImagePullSecrets is a list of references to secrets in the same namespace to use for pulling any images in pods that reference this ServiceAccount. ImagePullSecrets are distinct from Secrets because Secrets can be mounted in the pod, but ImagePullSecrets are only accessed by the kubelet. More info: https://kubernetes.io/docs/concepts/containers/images/#specifying-imagepullsecrets-on-a-pod  # noqa: E501

        :param image_pull_secrets: The image_pull_secrets of this V1ServiceAccount.  # noqa: E501
        :type image_pull_secrets: list[V1LocalObjectReference]
        """

        self._image_pull_secrets = image_pull_secrets

    @property
    def kind(self):
        """Gets the kind of this V1ServiceAccount.  # noqa: E501

        Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds  # noqa: E501

        :return: The kind of this V1ServiceAccount.  # noqa: E501
        :rtype: str
        """
        return self._kind

    @kind.setter
    def kind(self, kind):
        """Sets the kind of this V1ServiceAccount.

        Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds  # noqa: E501

        :param kind: The kind of this V1ServiceAccount.  # noqa: E501
        :type kind: str
        """

        self._kind = kind

    @property
    def metadata(self):
        """Gets the metadata of this V1ServiceAccount.  # noqa: E501


        :return: The metadata of this V1ServiceAccount.  # noqa: E501
        :rtype: V1ObjectMeta
        """
        return self._metadata

    @metadata.setter
    def metadata(self, metadata):
        """Sets the metadata of this V1ServiceAccount.


        :param metadata: The metadata of this V1ServiceAccount.  # noqa: E501
        :type metadata: V1ObjectMeta
        """

        self._metadata = metadata

    @property
    def secrets(self):
        """Gets the secrets of this V1ServiceAccount.  # noqa: E501

        Secrets is a list of the secrets in the same namespace that pods running using this ServiceAccount are allowed to use. Pods are only limited to this list if this service account has a \"kubernetes.io/enforce-mountable-secrets\" annotation set to \"true\". The \"kubernetes.io/enforce-mountable-secrets\" annotation is deprecated since v1.32. Prefer separate namespaces to isolate access to mounted secrets. This field should not be used to find auto-generated service account token secrets for use outside of pods. Instead, tokens can be requested directly using the TokenRequest API, or service account token secrets can be manually created. More info: https://kubernetes.io/docs/concepts/configuration/secret  # noqa: E501

        :return: The secrets of this V1ServiceAccount.  # noqa: E501
        :rtype: list[V1ObjectReference]
        """
        return self._secrets

    @secrets.setter
    def secrets(self, secrets):
        """Sets the secrets of this V1ServiceAccount.

        Secrets is a list of the secrets in the same namespace that pods running using this ServiceAccount are allowed to use. Pods are only limited to this list if this service account has a \"kubernetes.io/enforce-mountable-secrets\" annotation set to \"true\". The \"kubernetes.io/enforce-mountable-secrets\" annotation is deprecated since v1.32. Prefer separate namespaces to isolate access to mounted secrets. This field should not be used to find auto-generated service account token secrets for use outside of pods. Instead, tokens can be requested directly using the TokenRequest API, or service account token secrets can be manually created. More info: https://kubernetes.io/docs/concepts/configuration/secret  # noqa: E501

        :param secrets: The secrets of this V1ServiceAccount.  # noqa: E501
        :type secrets: list[V1ObjectReference]
        """

        self._secrets = secrets

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1ServiceAccount):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1ServiceAccount):
            return True

        return self.to_dict() != other.to_dict()



from kubernetes_asyncio.client.api_client import ApiClient
from kubernetes_asyncio.client.models import V1APIResourceList
from kubernetes_asyncio.client.models import V1CertificateSigningRequest
from kubernetes_asyncio.client.models import V1CertificateSigningRequestList
from kubernetes_asyncio.client.models import V1Status
from multidict import CIMultiDictProxy
from typing import Any
from typing import Awaitable

class CertificatesV1Api:
    def __init__(self, api_client: ApiClient | None = None) -> None: ...
    def create_certificate_signing_request(self, body: V1CertificateSigningRequest, **kwargs: Any) -> Awaitable[V1CertificateSigningRequest]: ...
    def create_certificate_signing_request_with_http_info(self, body: V1CertificateSigningRequest, **kwargs: Any) -> Awaitable[tuple[V1CertificateSigningRequest, int, CIMultiDictProxy]]: ...
    def delete_certificate_signing_request(self, name: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_certificate_signing_request_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_certificate_signing_request(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_certificate_signing_request_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def get_api_resources(self, **kwargs: Any) -> Awaitable[V1APIResourceList]: ...
    def get_api_resources_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1APIResourceList, int, CIMultiDictProxy]]: ...
    def list_certificate_signing_request(self, **kwargs: Any) -> Awaitable[V1CertificateSigningRequestList]: ...
    def list_certificate_signing_request_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1CertificateSigningRequestList, int, CIMultiDictProxy]]: ...
    def patch_certificate_signing_request(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1CertificateSigningRequest]: ...
    def patch_certificate_signing_request_approval(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1CertificateSigningRequest]: ...
    def patch_certificate_signing_request_approval_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1CertificateSigningRequest, int, CIMultiDictProxy]]: ...
    def patch_certificate_signing_request_status(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1CertificateSigningRequest]: ...
    def patch_certificate_signing_request_status_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1CertificateSigningRequest, int, CIMultiDictProxy]]: ...
    def patch_certificate_signing_request_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1CertificateSigningRequest, int, CIMultiDictProxy]]: ...
    def read_certificate_signing_request(self, name: str, **kwargs: Any) -> Awaitable[V1CertificateSigningRequest]: ...
    def read_certificate_signing_request_approval(self, name: str, **kwargs: Any) -> Awaitable[V1CertificateSigningRequest]: ...
    def read_certificate_signing_request_approval_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1CertificateSigningRequest, int, CIMultiDictProxy]]: ...
    def read_certificate_signing_request_status(self, name: str, **kwargs: Any) -> Awaitable[V1CertificateSigningRequest]: ...
    def read_certificate_signing_request_status_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1CertificateSigningRequest, int, CIMultiDictProxy]]: ...
    def read_certificate_signing_request_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1CertificateSigningRequest, int, CIMultiDictProxy]]: ...
    def replace_certificate_signing_request(self, name: str, body: V1CertificateSigningRequest, **kwargs: Any) -> Awaitable[V1CertificateSigningRequest]: ...
    def replace_certificate_signing_request_approval(self, name: str, body: V1CertificateSigningRequest, **kwargs: Any) -> Awaitable[V1CertificateSigningRequest]: ...
    def replace_certificate_signing_request_approval_with_http_info(self, name: str, body: V1CertificateSigningRequest, **kwargs: Any) -> Awaitable[tuple[V1CertificateSigningRequest, int, CIMultiDictProxy]]: ...
    def replace_certificate_signing_request_status(self, name: str, body: V1CertificateSigningRequest, **kwargs: Any) -> Awaitable[V1CertificateSigningRequest]: ...
    def replace_certificate_signing_request_status_with_http_info(self, name: str, body: V1CertificateSigningRequest, **kwargs: Any) -> Awaitable[tuple[V1CertificateSigningRequest, int, CIMultiDictProxy]]: ...
    def replace_certificate_signing_request_with_http_info(self, name: str, body: V1CertificateSigningRequest, **kwargs: Any) -> Awaitable[tuple[V1CertificateSigningRequest, int, CIMultiDictProxy]]: ...
